@@ -27,6 +27,7 @@ import isla.helpers
 import isla.isla_shortcuts as sc
 import isla.three_valued_truth
 from isla import language
+from isla.evaluator import evaluate
 from isla.existential_helpers import insert_tree, DIRECT_EMBEDDING, SELF_EMBEDDING, CONTEXT_ADDITION
 from isla.fuzzer import GrammarFuzzer, GrammarCoverageFuzzer
 from isla.helpers import delete_unreachable, dict_of_lists_to_list_of_dicts, \
@@ -37,6 +38,7 @@ from isla.language import VariablesCollector, split_conjunction, split_disjuncti
     convert_to_dnf, convert_to_nnf, ensure_unique_bound_variables, parse_isla, get_conjuncts, QuantifiedFormula
 from isla.derivation_tree import DerivationTree
 from isla.parser import EarleyParser
+from isla.three_valued_truth import ThreeValuedTruth
 from isla.type_defs import Grammar, Path
 from isla.z3_helpers import z3_solve, z3_subst, z3_eq
 
@@ -299,6 +301,16 @@ class ISLaSolver:
             for state in initial_states:
                 self.costs[state] = self.compute_cost(state)
 
+    def evaluate(
+            self,
+            inp: DerivationTree | str,
+            structural_predicates: Set[language.StructuralPredicate] = STANDARD_STRUCTURAL_PREDICATES,
+            semantic_predicates: Set[language.SemanticPredicate] = STANDARD_SEMANTIC_PREDICATES) -> ThreeValuedTruth:
+        if isinstance(inp, str):
+            inp = self.parse(inp)
+        assert isinstance(inp, DerivationTree)
+        return evaluate(self.formula, inp, self.grammar, structural_predicates, semantic_predicates)
+
     def solve(self) -> Generator[DerivationTree, None, None]:
         start_time = int(time.time())
         fuzzer = self.fuzzer_factory(self.grammar)
@@ -1316,7 +1328,7 @@ class ISLaSolver:
                 tree_substitutions.get(constant, constant):
                     (
                         val := maybe_model[z3.String(constant.name)].as_string(),
-                        DerivationTree(val, []) if constant.is_numeric() else self.parse(constant.n_type, val)
+                        DerivationTree(val, []) if constant.is_numeric() else self.parse(val, constant.n_type)
                     )[-1]
                 for constant in constants
             }
@@ -1604,10 +1616,12 @@ class ISLaSolver:
 
         return z3_regex
 
-    def parse(self, nonterminal: str, input: str) -> DerivationTree:
+    def parse(self, input: str, nonterminal: str = '<start>') -> DerivationTree:
         grammar = copy.deepcopy(self.grammar)
-        grammar["<start>"] = [nonterminal]
-        delete_unreachable(grammar)
+        if nonterminal != '<start>':
+            grammar["<start>"] = [nonterminal]
+            delete_unreachable(grammar)
+
         parser = EarleyParser(grammar)
         return DerivationTree.from_parse_tree(list(parser.parse(input))[0][1][0])
 
